@@ -11,13 +11,13 @@ Checking url, like:
 http://10.32.8.164:8080/DataService?requestType=getLatestDataName&directory=ECMWF_HR/TMP/850&fileName=&filter=*.024
 """
 
-from pickle import FALSE
 import warnings
 import re
 import http.client
 import urllib.parse
 import pickle
 import bz2
+import zlib
 from io import BytesIO
 from datetime import datetime, timedelta
 import numpy as np
@@ -137,7 +137,7 @@ def get_latest_initTime(directory, suffix="*.006"):
         print('Can not retrieve data from ' + directory)
         return None
     StringResult = DataBlock_pb2.StringResult()
-    if status == 200:
+    if status == 200:     # Standard response for successful HTTP requests
         StringResult.ParseFromString(response)
         if StringResult is not None:
             filename = StringResult.name
@@ -147,9 +147,8 @@ def get_latest_initTime(directory, suffix="*.006"):
                 return filename.split('.')[0]
         else:
             return None
-
-    # extract initial time
-    return filename.split(".")[0]
+    else:
+        return None
 
 
 def get_model_grid(directory, filename=None, suffix="*.024",
@@ -272,9 +271,9 @@ def get_model_grid(directory, filename=None, suffix="*.024",
                 if data_type == 4:
                     data = np.full((nmem, nlat, nlon), np.nan)
                 else:
-                    data = np.full((2, nmem, nlat, nlon), np.nan)
+                    data = np.full((nmem, 2, nlat, nlon), np.nan)
                 ind = 0
-                for imem in range(nmem):
+                for _ in range(nmem):
                     head_info_mem = np.frombuffer(
                         byteArray[ind:(ind+278)], dtype=head_dtype)
                     ind += 278
@@ -285,7 +284,7 @@ def get_model_grid(directory, filename=None, suffix="*.024",
                     if data_type == 4:
                         data[number, :, :] = np.squeeze(data_mem['data'])
                     else:
-                        data[:, number, :, :] = np.squeeze(data_mem['data'])
+                        data[number, :, :, :] = np.squeeze(data_mem['data'])
 
             # scale and offset the data, if necessary.
             if scale_off is not None:
@@ -340,18 +339,18 @@ def get_model_grid(directory, filename=None, suffix="*.024",
                                 'lat':lat_coord, 'lon':lon_coord})
                 else:
                     if level[0] == 0:
-                        data = data[np.newaxis, ...]
+                        data = data[:, np.newaxis, ...]
                         data = xr.Dataset({
-                            varname:(['time', 'number', 'lat', 'lon'], data, varattrs)},
+                            varname:(['number', 'time', 'lat', 'lon'], data, varattrs)},
                             coords={
-                                'time':time_coord, 'number':number_coord, 
+                                'number':number_coord, 'time':time_coord,
                                 'lat':lat_coord, 'lon':lon_coord})
                     else:
-                        data = data[np.newaxis, :, np.newaxis, ...]
+                        data = data[:, np.newaxis, np.newaxis, ...]
                         data = xr.Dataset({
-                            varname:(['time', 'number', 'level', 'lat', 'lon'], data, varattrs)},
+                            varname:(['number', 'time', 'level', 'lat', 'lon'], data, varattrs)},
                             coords={
-                                'time':time_coord, 'number':number_coord, 'level':level_coord, 
+                                'number':number_coord, 'time':time_coord, 'level':level_coord, 
                                 'lat':lat_coord, 'lon':lon_coord})
             elif data_type == 11:
                 speedattrs = {'long_name':'wind speed', 'units':'m/s'}
@@ -379,32 +378,32 @@ def get_model_grid(directory, filename=None, suffix="*.024",
                             coords={'lon': lon_coord, 'lat': lat_coord, 
                                     'level': level_coord, 'time': time_coord})
                 else:
-                    speed = np.squeeze(data[0, :, :, :])
-                    angle = np.squeeze(data[1, :, :, :])
+                    speed = np.squeeze(data[:, 0, :, :])
+                    angle = np.squeeze(data[:, 1, :, :])
                     # 原始数据文件中存储为: 西风为0度，南风为90度，东风为180度，北风为270度
                     # 修改为气象风向常规定义: 北方为0度, 东风为90度, 南风为180度, 西方为270度
                     angle = 270. - angle
                     angle[angle<0] = angle[angle<0] + 360.
                     if level[0] == 0:
-                        speed = speed[np.newaxis, ...]
-                        angle = angle[np.newaxis, ...]
+                        speed = speed[:, np.newaxis, ...]
+                        angle = angle[:, np.newaxis, ...]
                         data = xr.Dataset({
                             'speed': (
-                                ['time', 'number', 'lat', 'lon'], speed, speedattrs),
+                                ['number', 'time', 'lat', 'lon'], speed, speedattrs),
                             'angle': (
-                                ['time', 'number', 'lat', 'lon'], angle, angleattrs)},
+                                ['number', 'time', 'lat', 'lon'], angle, angleattrs)},
                             coords={
                                 'lon': lon_coord, 'lat': lat_coord,
                                 'number': number_coord, 'time': time_coord})
                     else:
-                        speed = speed[np.newaxis, :, np.newaxis, ...]
-                        angle = angle[np.newaxis, :, np.newaxis, ...]
+                        speed = speed[:, np.newaxis, np.newaxis, ...]
+                        angle = angle[:, np.newaxis, np.newaxis, ...]
                         data = xr.Dataset({
                             'speed': (
-                                ['time', 'number', 'level', 'lat', 'lon'],
+                                ['number', 'time', 'level', 'lat', 'lon'],
                                 speed, speedattrs),
                             'angle': (
-                                ['time', 'number', 'level', 'lat', 'lon'],
+                                ['number', 'time', 'level', 'lat', 'lon'],
                                 angle, angleattrs)},
                             coords={
                                 'lon': lon_coord, 'lat': lat_coord, 'level': level_coord,
@@ -601,7 +600,8 @@ def get_model_profiles(directory, filenames, levels, points, **kargs):
         return None
 
 
-def get_station_data(directory, filename=None, suffix="*.000", dropna=True, cache=True, cache_clear=True):
+def get_station_data(directory, filename=None, suffix="*.000",
+                     dropna=True, cache=True, cache_clear=True):
     """
     Retrieve station data from MICAPS cassandra service.
 
@@ -610,6 +610,7 @@ def get_station_data(directory, filename=None, suffix="*.000", dropna=True, cach
     :param suffix: the filename filter pattern which will
                    be used to find the specified file.
     :param dropna: the column which values is all na will be dropped.
+    :param limit: subset station data in the limit [lon0, lon1, lat0, lat1]
     :param cache: cache retrieved data to local directory, default is True.
     :return: pandas DataFrame.
 
@@ -639,7 +640,8 @@ def get_station_data(directory, filename=None, suffix="*.000", dropna=True, cach
 
     # retrieve data from cached file
     if cache:
-        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
+        cache_file = CONFIG.get_cache_file(
+            directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
         if cache_file.is_file():
             with open(cache_file, 'rb') as f:
                 data = pickle.load(f)
@@ -746,7 +748,8 @@ def get_station_data(directory, filename=None, suffix="*.000", dropna=True, cach
                 '801': 'Dewpoint', '803': 'Dewpoint_depression', '805': 'Relative_humidity',
                 '807': 'Relative_humidity_min', '809': 'Relative_humidity_day_avg', 
                 '811': 'Water_vapor_pressure', '813': 'Water_vapor_pressure_day_avg',
-                '1001': 'Rain', '1003': 'Rain_1h', '1005': 'Rain_3h', '1007': 'Rain_6h', '1009': 'Rain_12h', '1013': 'Rain_day',
+                '1001': 'Rain', '1003': 'Rain_1h', '1005': 'Rain_3h', '1007': 'Rain_6h', 
+                '1009': 'Rain_12h', '1011': 'Rain_24h', '1013': 'Rain_day',
                 '1015': 'Rain_20-08', '1017': 'Rain_08-20', '1019': 'Rain_20-20', '1021': 'Rain_08-08',
                 '1023': 'Evaporation', '1025': 'Evaporation_large', '1027': 'Precipitable_water',
                 '1201': 'Vis_1min', '1203': 'Vis_10min', '1205': 'Vis_min', '1207': 'Vis_manual',
@@ -791,7 +794,7 @@ def get_station_dataset(directory, filenames, allExists=True, pbar=False, **karg
         tqdm_filenames = tqdm(filenames, desc=directory + ": ")
     else:
         tqdm_filenames = filenames
-    for filename in filenames:
+    for filename in tqdm_filenames:
         data = get_station_data(directory, filename=filename, **kargs)
         if data:
             dataset.append(data)
@@ -818,7 +821,7 @@ def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True, c
     :return: satellite information and data.
 
     :Examples:
-    >>> directory = "SATELLITE/FY2E/L1/IR1/EQUAL"
+    >>> directory = "SATELLITE/FY4A/L1/CHINA/C004"
     >>> data = get_fy_awx(directory)
     """
 
@@ -865,7 +868,7 @@ def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True, c
                 print('There is no data ' + filename + ' in ' + directory)
                 return None
 
-            # the first class file head
+            # the first class file head  一级文件头记录采用定长方式, 共40字节
             head1_dtype = [
                 ('SAT96', 'S12'),                    # SAT96 filename
                 ('byteSequence', 'i2'),              # 整型数的字节顺序, 0 低字节在前, 高字节在后; !=0 高字节在前, 低字节在后.
@@ -883,7 +886,7 @@ def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True, c
             ind = 40
 
             if head1_info['productCategory']:
-                # the second class file head
+                # the second class file head  二级文件头采用不定长方式，内容依据产品的不同而不同.
                 head2_dtype = [
                     ('satelliteName', 'S8'),                 # 卫星名
                     ('year', 'i2'), ('month', 'i2'),
@@ -958,12 +961,15 @@ def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True, c
                 ind += head1_info['padDataLength'][0]
 
                  # retrieve data records
-                data_len = (head1_info['dataRecordNumber'][0].astype(np.int) *
+                data_len = (head1_info['dataRecordNumber'][0].astype(int) *
                             head1_info['recordLength'][0])
                 data = np.frombuffer(byteArray[ind:(ind + data_len)], dtype='u1', count=data_len)
                 if calibration_table is not None:
                     data = calibration_table[data]
                 data.shape = (head1_info['dataRecordNumber'][0], head1_info['recordLength'][0])
+                
+                # 由于数据是按照左上角开始放置, 为此需要对纬度顺序进行反转
+                data = np.flip(data, axis=0)
 
                 # construct longitude and latitude coordinates
                 # if use the verticalResolution and horizontalResolution, lon and lat will not be correct.
@@ -1060,7 +1066,34 @@ def get_fy_awxs(directory, filenames, allExists=True, pbar=False, **kargs):
     return xr.concat(dataset, dim='time')
 
 
-def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True, cache_clear=True):
+def _lzw_decompress(compressed):
+    """Decompress a list of output ks to a string.
+    refer to https://stackoverflow.com/questions/6834388/basic-lzw-compression-help-in-python.
+    """
+
+    # Build the dictionary.
+    dict_size = 256
+    dictionary = {chr(i): chr(i) for i in range(dict_size)}
+
+    w = result = compressed.pop(0)
+    for k in compressed:
+        if k in dictionary:
+            entry = dictionary[k]
+        elif k == dict_size:
+            entry = w + w[0]
+        else:
+            raise ValueError('Bad compressed k: %s' % k)
+        result += entry
+
+        # Add w+entry[0] to the dictionary.
+        dictionary[dict_size] = w + entry[0]
+        dict_size += 1
+
+        w = entry
+    return result
+
+
+def get_radar_mosaic(directory, filename=None, suffix="*.BIN", cache=True, cache_clear=True):
     """
     该程序主要用于读取和处理中国气象局CRaMS系统的雷达回波全国拼图数据.
 
@@ -1074,7 +1107,7 @@ def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True, ca
     :Example:
     >>> data = get_radar_mosaic("RADARMOSAIC/CREF/")
     >>> dir_dir = "RADARMOSAIC/CREF/"
-    >>> filename = "ACHN.CREF000.20201021.000000.LATLON"
+    >>> filename = "ACHN_CREF_20210413_005000.BIN"
     >>> CREF = get_radar_mosaic(dir_dir, filename=filename, cache=False)
     >>> print(CREF['time'].values)
     """
@@ -1122,94 +1155,196 @@ def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True, ca
                 print('There is no data ' + filename + ' in ' + directory)
                 return None
 
-            # define head structure
-            head_dtype = [
-                ('description', 'S128'),
-                # product name,  QREF=基本反射率, CREF=组合反射率,
-                # VIL=液态水含量, OHP=一小时降水
-                ('name', 'S32'),
-                ('organization', 'S16'),
-                ('grid_flag', 'u2'),  # 经纬网格数据标识，固定值19532
-                ('data_byte', 'i2'),  # 数据单元字节数，固定值2
-                ('slat', 'f4'),       # 数据区的南纬（度）
-                ('wlon', 'f4'),       # 数据区的西经（度）
-                ('nlat', 'f4'),       # 数据区的北纬（度）
-                ('elon', 'f4'),       # 数据区的东经（度）
-                ('clat', 'f4'),       # 数据区中心纬度（度）
-                ('clon', 'f4'),       # 数据区中心经度（度）
-                ('rows', 'i4'),       # 数据区的行数
-                ('cols', 'i4'),       # 每行数据的列数
-                ('dlat', 'f4'),       # 纬向分辨率（度）
-                ('dlon', 'f4'),       # 经向分辨率（度）
-                ('nodata', 'f4'),     # 无数据区的编码值
-                ('levelbybtes', 'i4'),  # 单层数据字节数
-                ('levelnum', 'i2'),   # 数据层个数
-                ('amp', 'i2'),        # 数值放大系数
-                ('compmode', 'i2'),   # 数据压缩存储时为1，否则为0
-                ('dates', 'u2'),      # 数据观测时间，为1970年1月1日以来的天数
-                ('seconds', 'i4'),    # 数据观测时间的秒数
-                ('min_value', 'i2'),  # 放大后的数据最小取值
-                ('max_value', 'i2'),  # 放大后的数据最大取值
-                ('reserved', 'i2', 6)  # 保留字节
-            ]
+            # check data format version
+            if byteArray[0:3] == b'MOC':
+                # the new V3 format
+                head_dtype = [
+                    ('label', 'S4'),         # 文件固定标识：MOC
+                    ('Version', 'S4'),       # 文件格式版本代码，如: 1.0，1.1，etc
+                    ('FileBytes', 'i4'),     # 包含头信息在内的文件字节数，不超过2M
+                    ('MosaicID', 'i2'),      # 拼图产品编号
+                    ('coordinate', 'i2'),    # 坐标类型: 2=笛卡儿坐标,3=等经纬网格坐标
+                    ('varname', 'S8'),       # 产品代码,如: ET,VIL,CR,CAP,OHP,OHPC
+                    ('description', 'S64'),  # 产品描述,如Composite Reflectivity mosaic
+                    ('BlockPos', 'i4'),      # 产品数据起始位置(字节顺序)
+                    ('BlockLen', 'i4'),      # 产品数据字节数
 
-            # read head information
-            head_info = np.frombuffer(byteArray[0:256], dtype=head_dtype)
-            ind = 256
+                    ('TimeZone', 'i4'),      # 数据时钟,0=世界时,28800=北京时
+                    ('yr', 'i2'),            # 观测时间中的年份
+                    ('mon', 'i2'),           # 观测时间中的月份（1－12）
+                    ('day', 'i2'),           # 观测时间中的日期（1－31）
+                    ('hr', 'i2'),            # 观测时间中的小时（00－23）
+                    ('min', 'i2'),           # 观测时间中的分（00－59）
+                    ('sec', 'i2'),           # 观测时间中的秒（00－59）
+                    ('ObsSeconds', 'i4'),    # 观测时间的seconds
+                    ('ObsDates', 'u2'),      # 观测时间中的Julian dates
+                    ('GenDates', 'u2'),      # 产品处理时间的天数
+                    ('GenSeconds', 'i4'),    # 产品处理时间的描述
 
-            # get data information
-            varname = head_info['name'][0].decode("utf-8", 'ignore').rsplit('\x00')[0]
-            longname = {'CREF': 'Composite Reflectivity', 'QREF': 'Basic Reflectivity',
-                        'VIL': 'Vertically Integrated Liquid', 'OHP': 'One Hour Precipitation'}
-            units = head_info['organization'][0].decode("utf-8", 'ignore').rsplit('\x00')[0]
-            amp = head_info['amp'][0]
+                    ('edge_s', 'i4'),        # 数据区的南边界，单位：1/1000度，放大1千倍
+                    ('edge_w', 'i4'),        # 数据区的西边界，单位：1/1000度，放大1千倍
+                    ('edge_n', 'i4'),        # 数据区的北边界，单位：1/1000度，放大1千倍
+                    ('edge_e', 'i4'),        # 数据区的东边界，单位：1/1000度，放大1千倍
+                    ('cx', 'i4'),            # 数据区中心坐标，单位：1/1000度，放大1千倍
+                    ('cy', 'i4'),            # 数据区中心坐标，单位：1/1000度，放大1千倍
+                    ('nX', 'i4'),            # 格点坐标为列数
+                    ('nY', 'i4'),            # 格点坐标为行数
+                    ('dx', 'i4'),            # 格点坐标为列分辨率，单位：1/10000度，放大1万倍
+                    ('dy', 'i4'),            # 格点坐标为行分辨率，单位：1/10000度，放大1万倍
+                    ('height', 'i2'),        # 雷达高度
+                    ('Compress', 'i2'),      # 数据压缩标识, 0=无,1=bz2,2=zip,3=lzw
+                    ('num_of_radars', 'i4'), # 有多少个雷达进行了拼图
+                    ('UnZipBytes', 'i4'),    # 数据段压缩前的字节数
+                    ('scale', 'i2'),
+                    ('unUsed', 'i2'),
+                    ('RgnID', 'S8'),
+                    ('units', 'S8'),
+                    ('reserved', 'S60')
+                ]
 
-            # define data variable
-            rows = head_info['rows'][0]
-            cols = head_info['cols'][0]
-            dlat = head_info['dlat'][0]
-            dlon = head_info['dlon'][0]
-            data = np.full(rows*cols, -9999, dtype=np.int32)
+                # read head information
+                head_info = np.frombuffer(byteArray[0:256], dtype=head_dtype)
+                ind = 256
 
-            # put data into array
-            while ind < len(byteArray):
-                irow = np.frombuffer(byteArray[ind:(ind + 2)], dtype='i2')[0]
-                ind += 2
-                icol = np.frombuffer(byteArray[ind:(ind + 2)], dtype='i2')[0]
-                ind += 2
-                if irow == -1 or icol == -1:
-                    break
-                nrec = np.frombuffer(byteArray[ind:(ind + 2)], dtype='i2')[0]
-                ind += 2
-                recd = np.frombuffer(
-                    byteArray[ind:(ind + 2*nrec)], dtype='i2', count=nrec)
-                ind += 2*nrec
-                position = (irow-1)*cols+icol-1
-                data[position:(position+nrec)] = recd
+                # get data information
+                varname = head_info['varname'][0]
+                longname = head_info['description'][0]
+                units = head_info['units'][0]
 
-            # reshape data
-            data.shape = (rows, cols)
+                # define data variable
+                rows = head_info['nY'][0]
+                cols = head_info['nX'][0]
+                dlat = head_info['dx'][0]/10000.
+                dlon = head_info['dy'][0]/10000.
 
-            # deal missing data and restore values
-            data = data.astype(np.float32)
-            data[data < 0] = np.nan
-            data /= amp
+                # decompress byte Array
+                # 目前主要支持bz2压缩格式, zip, lzw格式还没有测试过
+                if   head_info['Compress'] == 0:    # 无压缩
+                    data = np.frombuffer(byteArray[ind:], 'i2')
+                elif head_info['Compress'] == 1:    # 
+                    data = np.frombuffer(bz2.decompress(byteArray[ind:]), 'i2')
+                elif head_info['Compress'] == 2:
+                    data = np.frombuffer(zlib.decompress(byteArray[ind:]), 'i2')
+                elif head_info['Compress'] == 3:
+                    data = np.frombuffer(_lzw_decompress(byteArray[ind:]), 'i2')
+                else:
+                    print('Can not decompress data.')
+                    return None
+                
+                # reshape data
+                data.shape = (rows, cols)
 
-            # set longitude and latitude coordinates
-            lat = head_info['nlat'][0] - np.arange(rows)*dlat - dlat/2.0
-            lon = head_info['wlon'][0] + np.arange(cols)*dlon - dlon/2.0
+                # deal missing data and restore values
+                data = data.astype(np.float32)
+                data[data < 0] = np.nan
+                data /= head_info['scale'][0]
 
-            # reverse latitude axis
-            data = np.flip(data, 0)
-            lat = lat[::-1]
+                # set longitude and latitude coordinates
+                lat = head_info['edge_n'][0]/1000. - np.arange(rows)*dlat - dlat/2.0
+                lon = head_info['edge_w'][0]/1000. + np.arange(cols)*dlon - dlon/2.0
 
-            # set time coordinates
-            # 直接使用时间有问题, 需要天数减去1, 秒数加上28800(8小时)
-            time = datetime(1970, 1, 1) + timedelta(
-                days=head_info['dates'][0].astype(np.float64)-1,
-                seconds=head_info['seconds'][0].astype(np.float64)+28800)
-            time = np.array([time], dtype='datetime64[m]')
-            data = np.expand_dims(data, axis=0)
+                # reverse latitude axis
+                data = np.flip(data, 0)
+                lat = lat[::-1]
+
+                # set time coordinates
+                # 直接使用时间有问题, 需要天数减去1, 秒数加上28800(8小时)
+                time = datetime(head_info['yr'][0], head_info['mon'][0], head_info['day'][0],
+                                head_info['hr'][0], head_info['min'][0], head_info['sec'][0])
+                time = np.array([time], dtype='datetime64[m]')
+                data = np.expand_dims(data, axis=0)
+
+            else:
+                # the old LONLAT format.
+                # define head structure
+                head_dtype = [
+                    ('description', 'S128'),
+                    # product name,  QREF=基本反射率, CREF=组合反射率,
+                    # VIL=液态水含量, OHP=一小时降水
+                    ('name', 'S32'),
+                    ('organization', 'S16'),
+                    ('grid_flag', 'u2'),  # 经纬网格数据标识，固定值19532
+                    ('data_byte', 'i2'),  # 数据单元字节数，固定值2
+                    ('slat', 'f4'),       # 数据区的南纬（度）
+                    ('wlon', 'f4'),       # 数据区的西经（度）
+                    ('nlat', 'f4'),       # 数据区的北纬（度）
+                    ('elon', 'f4'),       # 数据区的东经（度）
+                    ('clat', 'f4'),       # 数据区中心纬度（度）
+                    ('clon', 'f4'),       # 数据区中心经度（度）
+                    ('rows', 'i4'),       # 数据区的行数
+                    ('cols', 'i4'),       # 每行数据的列数
+                    ('dlat', 'f4'),       # 纬向分辨率（度）
+                    ('dlon', 'f4'),       # 经向分辨率（度）
+                    ('nodata', 'f4'),     # 无数据区的编码值
+                    ('levelbybtes', 'i4'),  # 单层数据字节数
+                    ('levelnum', 'i2'),   # 数据层个数
+                    ('amp', 'i2'),        # 数值放大系数
+                    ('compmode', 'i2'),   # 数据压缩存储时为1，否则为0
+                    ('dates', 'u2'),      # 数据观测时间，为1970年1月1日以来的天数
+                    ('seconds', 'i4'),    # 数据观测时间的秒数
+                    ('min_value', 'i2'),  # 放大后的数据最小取值
+                    ('max_value', 'i2'),  # 放大后的数据最大取值
+                    ('reserved', 'i2', 6)  # 保留字节
+                ]
+
+                # read head information
+                head_info = np.frombuffer(byteArray[0:256], dtype=head_dtype)
+                ind = 256
+
+                # get data information
+                varname = head_info['name'][0].decode("utf-8", 'ignore').rsplit('\x00')[0]
+                longname = {'CREF': 'Composite Reflectivity', 'QREF': 'Basic Reflectivity',
+                            'VIL': 'Vertically Integrated Liquid', 'OHP': 'One Hour Precipitation'}
+                longname = longname.get(varname, 'radar mosaic')
+                units = head_info['organization'][0].decode("utf-8", 'ignore').rsplit('\x00')[0]
+                amp = head_info['amp'][0]
+
+                # define data variable
+                rows = head_info['rows'][0]
+                cols = head_info['cols'][0]
+                dlat = head_info['dlat'][0]
+                dlon = head_info['dlon'][0]
+                data = np.full(rows*cols, -9999, dtype=np.int32)
+
+                # put data into array
+                while ind < len(byteArray):
+                    irow = np.frombuffer(byteArray[ind:(ind + 2)], dtype='i2')[0]
+                    ind += 2
+                    icol = np.frombuffer(byteArray[ind:(ind + 2)], dtype='i2')[0]
+                    ind += 2
+                    if irow == -1 or icol == -1:
+                        break
+                    nrec = np.frombuffer(byteArray[ind:(ind + 2)], dtype='i2')[0]
+                    ind += 2
+                    recd = np.frombuffer(
+                        byteArray[ind:(ind + 2*nrec)], dtype='i2', count=nrec)
+                    ind += 2*nrec
+                    position = (irow-1)*cols+icol-1
+                    data[position:(position+nrec)] = recd
+
+                # reshape data
+                data.shape = (rows, cols)
+
+                # deal missing data and restore values
+                data = data.astype(np.float32)
+                data[data < 0] = np.nan
+                data /= amp
+
+                # set longitude and latitude coordinates
+                lat = head_info['nlat'][0] - np.arange(rows)*dlat - dlat/2.0
+                lon = head_info['wlon'][0] + np.arange(cols)*dlon - dlon/2.0
+
+                # reverse latitude axis
+                data = np.flip(data, 0)
+                lat = lat[::-1]
+
+                # set time coordinates
+                # 直接使用时间有问题, 需要天数减去1, 秒数加上28800(8小时)
+                time = datetime(1970, 1, 1) + timedelta(
+                    days=head_info['dates'][0].astype(np.float64)-1,
+                    seconds=head_info['seconds'][0].astype(np.float64)+28800)
+                time = np.array([time], dtype='datetime64[m]')
+                data = np.expand_dims(data, axis=0)
 
             # define coordinates
             time_coord = ('time', time)
@@ -1221,8 +1356,7 @@ def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True, ca
                 '_CoordinateAxisType':'Lat', "axis": "Y"})
 
             # create xarray
-            varattrs = {'long_name': longname.get(varname, 'radar mosaic'), 
-                        'short_name': varname, 'units': units}
+            varattrs = {'long_name': longname, 'short_name': varname, 'units': units}
             data = xr.Dataset({'data':(['time', 'lat', 'lon'], data, varattrs)},
                 coords={'time':time_coord, 'lat':lat_coord, 'lon':lon_coord})
 
